@@ -12,6 +12,7 @@
 #include <string>
 #include <list>
 #include <vector>
+#include <ctime>
 
 
 namespace XML
@@ -70,6 +71,12 @@ namespace DBTREE
     // 板移転
     const bool move_board( const std::string& url_old, const std::string& url_new );
 
+    // 板移転情報の保存の有効切り替え
+    void set_enable_save_movetable( const bool set );
+
+    // 移転情報保存
+    void save_movetable();
+
     // bbslist系
     const XML::Document& get_xml_document();
     const std::list< DBTREE::ETCBOARDINFO >& get_etcboards();
@@ -109,7 +116,7 @@ namespace DBTREE
     void board_download_subject( const std::string& url, const std::string& url_update_view );
     const bool board_is_loading( const std::string& url );
     void board_stop_load( const std::string& url );
-    std::list< DBTREE::ArticleBase* >& board_list_subject( const std::string& url );
+    std::vector< DBTREE::ArticleBase* >& board_list_subject( const std::string& url );
     const int board_view_sort_column( const std::string& url );
     void board_set_view_sort_column( const std::string& url, int column );
     const int board_view_sort_mode( const std::string& url );
@@ -173,9 +180,12 @@ namespace DBTREE
     void read_boardinfo_all();
 
     // キャッシュ内のログ検索
-    // datファイルのURL(read.cgi型)を返す
-    std::list< std::string > search_cache_all( const std::string& url, const std::string& query, bool mode_or, bool& stop );
-    std::list< std::string > search_cache( const std::string& url, const std::string& query, bool mode_or, bool& top );
+    // ArticleBase のアドレスをリスト(list_article)にセットして返す
+    // query が空の時はキャッシュにあるログを全てヒットさせる
+    void search_cache_all( std::vector< DBTREE::ArticleBase* >& list_article,
+                           const std::string& query, const bool mode_or, const bool& stop );
+    void search_cache( const std::string& url, std::vector< DBTREE::ArticleBase* >& list_article,
+                       const std::string& query, const bool mode_or, const bool& stop );
     
     // article 系
     const bool article_is_cached( const std::string& url ); // キャッシュにあるかどうか
@@ -200,6 +210,7 @@ namespace DBTREE
     void article_set_number_seen( const std::string& url, int seen );
     const int article_number_new( const std::string& url );    
     const bool article_is_loading( const std::string& url );
+    const bool article_is_checking_update( const std::string& url );
     void article_download_dat( const std::string& url, const bool check_update );
     void article_set_url_pre_article( const std::string& url, const std::string& url_pre_article );
     void article_copy_article_info( const std::string& url, const std::string& url_src );
@@ -294,6 +305,7 @@ namespace DBTREE
     // スレあぼーん情報
     // グローバルなあぼーん情報は globalconf が管理
     const std::list< std::string >& get_abone_list_thread( const std::string& url );
+    const std::list< std::string >& get_abone_list_thread_remove( const std::string& url );
     const std::list< std::string >& get_abone_list_word_thread( const std::string& url );
     const std::list< std::string >& get_abone_list_regex_thread( const std::string& url );
     const std::vector< char >& get_abone_vec_res( const std::string& url );
@@ -307,12 +319,13 @@ namespace DBTREE
     // CONFIG::set_abone_number_thread() などでグローバル設定をした後などに呼び出す
     void update_abone_thread();
 
-    // スレあぼーん状態のリセット(情報セットとスレ一覧の表示更新を同時におこなう)
+    // スレあぼーん状態のリセット
+    // redraw : スレ一覧の表示更新を行う    
     void reset_abone_thread( const std::string& url,
                              const std::list< std::string >& threads,
                              const std::list< std::string >& words,
                              const std::list< std::string >& regexs,
-                             const int number, const int hour );
+                             const int number, const int hour, const bool redraw );
 
     //
     // 各articlebase別のあぼーん情報
