@@ -54,14 +54,25 @@ enum
 };
 
 
+// レジュームのモード
+enum
+{
+    RESUME_NO = 0,  // レジューム無し
+    RESUME_MODE1,   // 通常のレジューム
+    RESUME_MODE2,   // サーバが range を無視してデータを送ってきた
+    RESUME_MODE3,   // サーバが range を無視してデータを送ってきた時にデータをスキップ中
+    RESUME_FAILED   // レジューム失敗
+};
+
+
 using namespace DBTREE;
 
 
-NodeTreeBase::NodeTreeBase( const std::string url, const std::string& modified )
+NodeTreeBase::NodeTreeBase( const std::string& url, const std::string& modified )
     : SKELETON::Loadable(),
       m_url( url ),
       m_lng_dat( 0 ),
-      m_resume ( 0 ),
+      m_resume ( RESUME_NO ),
       m_broken( 0 ),
       m_heap( SIZE_OF_HEAP ),
       m_buffer_lines( NULL ),
@@ -69,6 +80,7 @@ NodeTreeBase::NodeTreeBase( const std::string url, const std::string& modified )
       m_buffer_write( NULL ),
       m_check_update( false ),
       m_check_write( false ),
+      m_loading_newthread( false ),
       m_fout ( 0 )
 {
     set_date_modified( modified );
@@ -94,7 +106,7 @@ NodeTreeBase::NodeTreeBase( const std::string url, const std::string& modified )
     m_num_id[ LINK_LOW ] = CONFIG::get_num_id_low();
 
 #ifdef _DEBUG
-    std::cout << "NodeTreeBase::NodeTreeBase url = " << m_url << " modified = " << date_modified()
+    std::cout << "NodeTreeBase::NodeTreeBase url = " << m_url << " modified = " << get_date_modified()
               << " noname = " << m_default_noname << std::endl;
 #endif
 }
@@ -360,12 +372,20 @@ std::list< int > NodeTreeBase::get_res_with_url()
 
 
 //
-// number番のレスを参照しているレスの番号全てをリストにして取得
+// res_num に含まれるレスを参照しているレス番号をリストにして取得
 //
-std::list< int > NodeTreeBase::get_res_reference( int number )
+std::list< int > NodeTreeBase::get_res_reference( const std::list< int >& res_num )
 {
     std::list< int > list_resnum;
-    for( int i = number + 1; i <= m_id_header; ++i ){
+
+    if( ! res_num.size() ) return list_resnum;
+
+    std::list< int >::const_iterator it_res = res_num.begin();
+
+    for( int i = ( *it_res ) + 1; i <= m_id_header; ++i ){
+
+        //  透明あぼーんは除外
+        if( m_abone_transparent && get_abone( i ) ) continue;
 
         NODE* head = res_header( i );
         if( head ){
@@ -391,16 +411,19 @@ std::list< int > NodeTreeBase::get_res_reference( int number )
                                 if( anc_from == 0 ) break;
                                 ++anc;
 
-                                // >>1-1000 みたいなアンカーは弾く
-                                if( anc_to - anc_from < RANGE_REF && anc_from <= number && number <= anc_to
-                                    && ( ! m_abone_transparent || ! get_abone( i ) ) //  透明あぼーんしていない　or あぼーんしていない
-                                    ) {
-                                    list_resnum.push_back( i );
-                                    block = BLOCK_NUM;
-                                    break;
+                                it_res = res_num.begin();
+                                for( ; it_res != res_num.end(); ++it_res ){
+
+                                    const int number = ( *it_res );
+
+                                    // >>1-1000 みたいなアンカーは弾く
+                                    if( anc_to - anc_from < RANGE_REF && anc_from <= number && number <= anc_to
+                                        ) {
+                                        list_resnum.push_back( i );
+                                        goto EXIT_LOOP;
+                                    }
                                 }
                             }
-                            if( anc_from != 0 ) break;
                         }
                     }
 
@@ -410,9 +433,11 @@ std::list< int > NodeTreeBase::get_res_reference( int number )
 
             } // for( block )
 
-        }
+        } // if( head )
 
-    }
+EXIT_LOOP:;
+
+    } // for( int i )
 
 #ifdef _DEBUG
     std::cout << "NodeTreeBase::get_reference\n";
@@ -937,6 +962,8 @@ void NodeTreeBase::load_cache()
             size_t size = 0;
             m_check_update = false;
             m_check_write = false;
+            m_loading_newthread = false;
+            set_resume( false );
             init_loading();
             const size_t str_length = str.length();
             while( size < str_length ){
@@ -945,6 +972,12 @@ void NodeTreeBase::load_cache()
                 size += size_tmp;
             }
             receive_finish();
+
+            // レジューム時のチェック用に生データの先頭から RESUME_CHKSIZE バイト分をコピーしておく
+            // 詳しくは NodeTreeBase::receive_data() を参照せよ
+            const size_t length_chk = MIN( RESUME_CHKSIZE, str_length );
+            memcpy( m_resume_head, data, length_chk );
+            m_resume_head[ length_chk ] = '\0';
         }
     }
 }
@@ -960,10 +993,22 @@ void NodeTreeBase::init_loading()
     // 一時バッファ作成
     if( ! m_buffer_lines ) m_buffer_lines = ( char* ) malloc( MAXSISE_OF_LINES ); 
     if( ! m_parsed_text ) m_parsed_text = ( char* ) malloc( MAXSISE_OF_LINES );
-    if( m_check_write && ! m_buffer_write ) m_buffer_write = ( char* ) malloc( MAXSISE_OF_LINES ); 
 }
 
 
+//
+// レジュームのモードをセットする
+//
+void NodeTreeBase::set_resume( const bool resume )
+{
+#ifdef _DEBUG
+    std::cout << "NodeTreeBase::set_resume resume = " << resume << std::endl;
+#endif
+
+    if( resume ) m_resume = RESUME_MODE1;
+    else m_resume = RESUME_NO;
+}
+
 
 //
 // ロード開始
@@ -975,13 +1020,13 @@ void NodeTreeBase::download_dat( const bool check_update )
     if( is_loading() ) return;
 
     m_check_update = check_update;
-    m_check_write = false;
-    if( ! m_check_update ) m_check_write = MESSAGE::get_log_manager()->has_items( m_url, ! get_res_number() );
+    m_check_write = ! m_check_update;
+    m_loading_newthread = ( ! get_res_number() );
 
 #ifdef _DEBUG    
     std::cout << "NodeTreeBase::download_dat : " << m_url << " lng = " << m_lng_dat << std::endl
-              << "modified = " << date_modified() << " check_update = " << check_update
-              << " check_write = " << m_check_write << std::endl;
+              << "modified = " << get_date_modified() << " check_update = " << check_update
+              << " newthread = " << m_loading_newthread << std::endl;
 #endif
 
 
@@ -1014,7 +1059,7 @@ void NodeTreeBase::download_dat( const bool check_update )
             std::cout << "open " << path_cache.c_str() << std::endl;
 #endif
 
-            m_fout = fopen( path_cache.c_str(), "ab" );
+            m_fout = fopen( to_locale_cstr( path_cache ), "ab" );
             if( m_fout == NULL ){
                 MISC::ERRMSG( "fopen failed : " + path_cache );
             }
@@ -1055,30 +1100,47 @@ void NodeTreeBase::receive_data( const char* data, size_t size )
 
 #ifdef _DEBUG
         std::cout << "NodeTreeBase::receive_data : code = " << get_code() << std::endl;
+        std::cout << data << std::endl;
 #endif
-
         return;
     }
 
     if( m_check_update ) return;
 
-    // レジューム処理
-    // レジュームした時に先頭が '\n' かチェック
-    // '\n'で無ければあぼーんがあったということ
-    if( m_resume ){
+    // 通常のレジューム処理
+    if( m_resume == RESUME_MODE1 ){
+
+#ifdef _DEBUG
+        std::cout << "resume mode = " << m_resume << " -> ";
+#endif
+
+        // レジュームした時に先頭が '\n' ならレジューム成功
+        if( data[ 0 ] == '\n' ){
 
-        if( data[ 0 ] != '\n' ){
-            m_broken = true;
-            MISC::ERRMSG( "failed to resume" );
-        }
-        else{
             ++data;
             --size;
+            m_resume = RESUME_NO;
         }
 
-        m_resume = false;
+        // サーバが range を無視してデータを送ってきた
+        // 続きは add_raw_lines() を参照
+        else if( get_code() == HTTP_OK ) m_resume = RESUME_MODE2;
+
+        // あぼーんが起きた?
+        else{
+
+            m_broken = true;
+            MISC::ERRMSG( "failed to resume" );
+            m_resume = RESUME_NO;
+        }
+
+#ifdef _DEBUG
+        std::cout << m_resume << std::endl;
+#endif
     }
 
+    if( m_resume == RESUME_FAILED ) return;
+
     if( !size ) return;
     
     // バッファが '\n' で終わるように調整
@@ -1109,6 +1171,7 @@ void NodeTreeBase::receive_finish()
         && get_code() != HTTP_OK
         && get_code() != HTTP_PARTIAL_CONTENT
         && get_code() != HTTP_NOT_MODIFIED
+        && get_code() != HTTP_OLD
         ){
         std::ostringstream err;
         err << m_url << std::endl
@@ -1129,7 +1192,7 @@ void NodeTreeBase::receive_finish()
         if( get_res_number() == 0 ){
 
             std::string path = CACHE::path_dat( m_url );
-            if( CACHE::file_exists( path ) == CACHE::EXIST_FILE ) unlink( path.c_str() );
+            if( CACHE::file_exists( path ) == CACHE::EXIST_FILE ) unlink( to_locale_cstr( path ) );
             set_date_modified( std::string() );
         }
 
@@ -1137,7 +1200,7 @@ void NodeTreeBase::receive_finish()
         if( !get_ext_err().empty() ) set_date_modified( std::string() );
 
         // 書き込みチェック終了
-        if( m_check_write
+        if( m_check_write && MESSAGE::get_log_manager()->size()
             && ( get_code() == HTTP_OK
                  || get_code() == HTTP_PARTIAL_CONTENT
                  || get_code() == HTTP_NOT_MODIFIED
@@ -1147,14 +1210,18 @@ void NodeTreeBase::receive_finish()
 
 #ifdef _DEBUG
     std::cout << "NodeTreeBase::receive_finish lng = " << m_lng_dat
+              << " raw lng = " << current_length()
               << " code = " << get_code() << " " << get_str_code()
-              << " modified = " << date_modified() << std::endl;
+              << " modified = " << get_date_modified() << std::endl;
 #endif    
     
     // 親 article クラスにシグナルを打ってツリー構造が変わったことを教える
     m_sig_finished.emit();
 
     clear();
+    m_check_update = false;
+    m_check_write = false;
+    m_loading_newthread = false;
 }
 
 
@@ -1175,10 +1242,59 @@ void NodeTreeBase::add_raw_lines( char* rawlines, size_t size )
     // 保存前にrawデータを加工
     rawlines = process_raw_lines( rawlines );
 
-    // キャッシュに保存
     size_t lng = strlen( rawlines );
-    if( !lng ) return;
+    if( ! lng ) return;
+
+    // サーバが range を無視してデータを送ってきたときのレジューム処理
+    if( m_resume == RESUME_MODE2 ){
+
+#ifdef _DEBUG
+        std::cout << "NodeTreeBase::add_raw_lines : resume\n";
+#endif
+
+        // 先頭からdatを送ってきたかチェック
+        const size_t length_chk = MIN( lng, MIN( RESUME_CHKSIZE, strlen( m_resume_head ) ) );
+        if( strncmp( rawlines, m_resume_head, length_chk ) == 0 ){
+            m_resume = RESUME_MODE3;
+            m_resume_lng = 0;
+        }
+
+        // 全く違うデータを送ってきた
+        else{
+
+            m_broken = true;
+            MISC::ERRMSG( "failed to resume" );
+            m_resume = RESUME_FAILED;
+            return;
+        }
+    }
+
+    // レジューム処理でデータをスキップ中
+    if( m_resume == RESUME_MODE3 ){
+
+#ifdef _DEBUG
+        std::cout << "NodeTreeBase::add_raw_lines : resume skip resume_lng = " << m_resume_lng
+                  << " lng = " << lng << " / lng_dat = " << m_lng_dat << std::endl;
+#endif
+
+        m_resume_lng += lng;
+        if( m_resume_lng <= m_lng_dat ) return;
+
+        // 越えた分をカットしてレジューム処理終了
+        rawlines += ( lng  - ( m_resume_lng - m_lng_dat ) );
+        lng = ( m_resume_lng - m_lng_dat );
+        m_resume = RESUME_NO;
+
+#ifdef _DEBUG
+        std::cout << "resume finished : lng = " << lng << std::endl;
+#endif
+    }
+
+    if( ! lng ) return;
+
     m_lng_dat += lng;
+
+    // キャッシュに保存
     if( m_fout ){
 #ifdef _DEBUG
         std::cout << "NodeTreeBase::add_raw_lines save " << lng << " bytes\n";
@@ -1308,7 +1424,9 @@ const char* NodeTreeBase::add_one_dat_line( const char* datline )
     }
 
     // 自分の書き込みかチェック
-    if( m_check_write ){
+    if( m_check_write && MESSAGE::get_log_manager()->has_items( m_url, m_loading_newthread ) ){
+
+        if( ! m_buffer_write ) m_buffer_write = ( char* ) malloc( MAXSISE_OF_LINES ); 
 
         // 簡易チェック
         // 最初の lng_check 文字だけ見る
@@ -1443,7 +1561,11 @@ void NodeTreeBase::parse_mail( NODE* header, const char* str, const int lng )
     int color = COLOR_CHAR;
     int i = 0;
     while( str[ i ] != 's' && i < lng ) ++i;
-    if( str[ i ] != 's' || str[ i+1 ] != 'a' || str[ i+2 ] != 'g' || str[ i+3 ] != 'e' ) color = COLOR_CHAR_AGE;
+    if( str[ i ] != 's' || str[ i+1 ] != 'a' || str[ i+2 ] != 'g' || str[ i+3 ] != 'e' ){
+        color = COLOR_CHAR_AGE;
+        header->headinfo->sage = FALSE;
+    }
+    else header->headinfo->sage = TRUE;
 
     header->headinfo->block[ BLOCK_MAIL ] = create_node_block();
     create_node_text( "[", color );
@@ -1454,8 +1576,7 @@ void NodeTreeBase::parse_mail( NODE* header, const char* str, const int lng )
     const bool sssp = false;
     parse_html( str, lng, color, digitlink, bold, ahref, sssp );
 
-    if( color == COLOR_CHAR ) create_node_text( "]", color );
-    else create_node_text( "]", color );
+    create_node_text( "]", color );
 }
 
 
@@ -1720,7 +1841,7 @@ void NodeTreeBase::parse_html( const char* str, const int lng, const int color_t
 
                 int n_in = 0;
                 int n_out = 0;
-                DBTREE::decode_char( "&#12539;", n_in, m_parsed_text + lng_text, n_out );
+                DBTREE::decode_char( "&#12539;", n_in, m_parsed_text + lng_text, n_out, false );
                 lng_text += n_out;
             }
 
@@ -1853,7 +1974,7 @@ void NodeTreeBase::parse_html( const char* str, const int lng, const int color_t
         // 特殊文字デコード
         if( *pos == '&' ){
 
-            int ret_decode = DBTREE::decode_char( pos, n_in, m_parsed_text + lng_text, n_out );
+            int ret_decode = DBTREE::decode_char( pos, n_in, m_parsed_text + lng_text, n_out, false );
 
             if( ret_decode != NODE_NONE ){
 
@@ -1912,6 +2033,10 @@ void NodeTreeBase::parse_write( const char* str, const int lng, const int max_ln
 {
     if( ! m_buffer_write ) return;
 
+#ifdef _DEBUG
+    std::cout << "NodeTreeBase::parse_write lng = " << lng << " max = " << max_lng_write << std::endl;
+#endif
+
     bool head = true;
     const char* pos = str;
     const char* pos_end = str + lng;
@@ -1960,6 +2085,15 @@ void NodeTreeBase::parse_write( const char* str, const int lng, const int max_ln
             continue;
         }
 
+        // quot
+        else if( *pos == '&' && ( *( pos + 1 ) == 'q' && *( pos + 2 ) == 'u' && *( pos + 3 ) == 'o' && *( pos + 4 ) == 't' && *( pos + 5 ) == ';' ) ){
+
+            *(pos_write++) = '"';
+            pos += 5;
+
+            continue;
+        }
+
         // 先頭のsssp
         else if( head && *pos == 's' && ( *( pos + 1 ) == 's' && *( pos + 2 ) == 's' && *( pos + 3 ) == 'p' ) ){
 
@@ -1969,6 +2103,15 @@ void NodeTreeBase::parse_write( const char* str, const int lng, const int max_ln
             continue;
         }
 
+        // 水平タブ
+        else if( *pos == '\t' ){
+
+            // 空白に置き換える
+            *(pos_write++) = ' ';
+
+            continue;
+        }
+
         head = false;
         *(pos_write++) = *pos;
     }
@@ -1993,8 +2136,8 @@ void NodeTreeBase::parse_write( const char* str, const int lng, const int max_ln
 //
 // 戻り値 : アンカーが現れれば true
 //
-bool NodeTreeBase::check_anchor( int mode, const char* str_in,
-                                 int& n_in, char* str_out, char* str_link, int lng_link, ANCINFO* ancinfo )
+const bool NodeTreeBase::check_anchor( const int mode, const char* str_in,
+                                       int& n_in, char* str_out, char* str_link, int lng_link, ANCINFO* ancinfo )
 {
     char tmp_out[ 64 ];
     int lng_out = 0;
@@ -2056,9 +2199,9 @@ bool NodeTreeBase::check_anchor( int mode, const char* str_in,
     }
 
     // 数字かチェック
-    unsigned int n, dig;
-    ancinfo->anc_from = ancinfo->anc_to = MISC::str_to_uint( pos, dig, n );
-    if( dig == 0 || dig > MAX_LINK_DIGIT || ancinfo->anc_from == 0 ){
+    size_t n, dig;
+    int num = MISC::str_to_uint( pos, dig, n );
+    if( dig == 0 || dig > MAX_LINK_DIGIT || num == 0 ){
 
         // モード2で数字が長すぎるときは飛ばす
         if( mode == 2 && dig > MAX_LINK_DIGIT ) n_in = ( int )( pos - str_in ) + n; 
@@ -2072,16 +2215,34 @@ bool NodeTreeBase::check_anchor( int mode, const char* str_in,
     memcpy( str_out, tmp_out, lng_out );
     memcpy( str_out + lng_out, pos, n );
     str_out[ lng_out + n ] = '\0';
-
-    // アンカー文字
-    snprintf( str_link, lng_link, "%d", ancinfo->anc_from );
     pos += n;
     lng_out += n;    
 
+    // </a>をキャンセル
+    if( *( pos ) == '<' && *( pos + 1 ) == '/' && ( *( pos + 2 ) == 'a' || *( pos + 2 ) == 'A' ) && *( pos + 3 ) == '>' ){
 
-    // dat形式では "&gt;&gt;数字"のパターンの場合には後に</a>がついてるのでのぞく
-    if( *( pos ) == '<' && *( pos + 1 ) == '/' && ( *( pos + 2 ) == 'a' || *( pos + 2 ) == 'A' ) && *( pos + 3 ) == '>' ) pos += 4;
+        pos += 4;
 
+        // もう一度数字チェック
+        // >>1１ を書き込むと <a href="..">&gt;&gt;1</a>１ となるため
+        size_t n2, dig2;
+        const int num2 = MISC::str_to_uint( pos, dig2, n2 );
+        if( dig2 > 0 && dig2 <= MAX_LINK_DIGIT ){
+
+            for( size_t i = 0; i < dig2; ++i ) num *= 10;
+            num += num2;
+
+            memcpy( str_out + lng_out, pos, n2 );
+            str_out[ lng_out + n2 ] = '\0';
+            pos += n2;
+            lng_out += n2;
+        }
+    }
+
+    ancinfo->anc_from = ancinfo->anc_to = num;
+
+    // アンカー文字
+    snprintf( str_link, lng_link, "%d", ancinfo->anc_from );
 
     // "-" でつながってる場合同じことをもう一回
     int offset = 0;
@@ -2151,37 +2312,24 @@ const int NodeTreeBase::check_link( const char* str_in, const int lng_in, int& n
     // リンクの長さを取得
     n_in = delim_pos;
     bool url_encode = false;
+    int n_in_tmp, n_out_tmp;
     char cchar = *( str_in + n_in );
+
     while(
+
         // バッファサイズを超えない
         n_in < lng_in
+
         // URLとして扱う文字かどうか
         && MISC::is_url_char( &cchar, loose_url )
+
         // HTML特殊文字は除く( &〜; )
         && ! (
             *( str_in + n_in ) == '&'
-            && (
-                 (
-                   // quot;
-                   *( str_in + n_in + 1 ) == 'q'
-                   && *( str_in + n_in + 2 ) == 'u'
-                   && *( str_in + n_in + 3 ) == 'o'
-                   && *( str_in + n_in + 4 ) == 't'
-                   && *( str_in + n_in + 5 ) == ';'
-                 ) || (
-                   // lt;
-                   *( str_in + n_in + 1 ) == 'l'
-                   && *( str_in + n_in + 2 ) == 't'
-                   && *( str_in + n_in + 3 ) == ';'
-                 ) || (
-                   // gt;
-                   *( str_in + n_in + 1 ) == 'g'
-                   && *( str_in + n_in + 2 ) == 't'
-                   && *( str_in + n_in + 3 ) == ';'
-                 )
+            && DBTREE::decode_char( str_in + n_in, n_in_tmp, NULL, n_out_tmp, true ) != DBTREE::NODE_NONE
             )
-        )
-    ){
+        ){
+
         // "^"と"|"はエンコードする
         // "[]"はダウンローダに渡す用途のためにエンコードしないでおく
         if( loose_url && ( cchar == '^' || cchar == '|' ) ) url_encode = true;
@@ -2273,12 +2421,12 @@ void NodeTreeBase::clear_abone()
 
 
 // あぼーん情報を親クラスのarticlebaseからコピーする
-void NodeTreeBase::copy_abone_info( std::list< std::string >& list_abone_id,
-                                    std::list< std::string >& list_abone_name,
-                                    std::list< std::string >& list_abone_word,
-                                    std::list< std::string >& list_abone_regex,
+void NodeTreeBase::copy_abone_info( const std::list< std::string >& list_abone_id,
+                                    const std::list< std::string >& list_abone_name,
+                                    const std::list< std::string >& list_abone_word,
+                                    const std::list< std::string >& list_abone_regex,
                                     const std::vector< char >& vec_abone_res,
-                                    const bool abone_transparent, const bool abone_chain )
+                                    const bool abone_transparent, const bool abone_chain, const bool abone_age )
 {
     m_list_abone_id = list_abone_id;
     m_list_abone_name = list_abone_name;
@@ -2305,6 +2453,8 @@ void NodeTreeBase::copy_abone_info( std::list< std::string >& list_abone_id,
 
     if( CONFIG::get_abone_chain() ) m_abone_chain = true;
     else m_abone_chain = abone_chain;
+
+    m_abone_age = abone_age;
 }
 
 
@@ -2332,7 +2482,7 @@ void NodeTreeBase::update_abone_all()
 //
 // from_number番から to_number 番までのレスのあぼーん状態を更新
 //
-void NodeTreeBase::update_abone( int from_number, int to_number )
+void NodeTreeBase::update_abone( const int from_number, const int to_number )
 {
     if( empty() ) return;
     if( to_number < from_number ) return;
@@ -2341,6 +2491,7 @@ void NodeTreeBase::update_abone( int from_number, int to_number )
         if( check_abone_res( i ) )  continue;
         if( check_abone_id( i ) )  continue;
         if( check_abone_name( i ) ) continue;
+        if( check_abone_mail( i ) ) continue;
         if( check_abone_word( i ) ) continue;
         if( check_abone_chain( i ) ) continue;
     }
@@ -2353,7 +2504,7 @@ void NodeTreeBase::update_abone( int from_number, int to_number )
 //
 // あぼーんの時はtrueを返す
 //
-bool NodeTreeBase::check_abone_res( int number )
+const bool NodeTreeBase::check_abone_res( const int number )
 {
     if( ! m_vec_abone_res.size() ) return false;
     if( ! m_vec_abone_res[ number ] ) return false;
@@ -2372,10 +2523,10 @@ bool NodeTreeBase::check_abone_res( int number )
 //
 // あぼーんの時はtrueを返す
 //
-bool NodeTreeBase::check_abone_id( int number )
+const bool NodeTreeBase::check_abone_id( const int number )
 {
-    bool check_id = ! m_list_abone_id.empty();
-    bool check_id_board = ! m_list_abone_id_board.empty();
+    const bool check_id = ! m_list_abone_id.empty();
+    const bool check_id_board = ! m_list_abone_id_board.empty();
 
     if( !check_id && !check_id_board ) return false;
 
@@ -2385,7 +2536,7 @@ bool NodeTreeBase::check_abone_id( int number )
     if( head->headinfo->abone ) return true;
     if( ! head->headinfo->block[ BLOCK_ID_NAME ] ) return false;
 
-    int ln_protoid = strlen( PROTO_ID );
+    const int ln_protoid = strlen( PROTO_ID );
 
     // ローカルID
     if( check_id ){
@@ -2422,11 +2573,11 @@ bool NodeTreeBase::check_abone_id( int number )
 //
 // あぼーんの時はtrueを返す
 //
-bool NodeTreeBase::check_abone_name( int number )
+const bool NodeTreeBase::check_abone_name( const int number )
 {
-    bool check_name = ! m_list_abone_name.empty();
-    bool check_name_board = ! m_list_abone_name_board.empty();
-    bool check_name_global = ! CONFIG::get_list_abone_name().empty();
+    const bool check_name = ! m_list_abone_name.empty();
+    const bool check_name_board = ! m_list_abone_name_board.empty();
+    const bool check_name_global = ! CONFIG::get_list_abone_name().empty();
 
     if( !check_name && !check_name_board && !check_name_global ) return false;
 
@@ -2436,8 +2587,8 @@ bool NodeTreeBase::check_abone_name( int number )
     if( head->headinfo->abone ) return true;
     if( ! head->headinfo->name ) return false;
 
-    std::list< std::string >::iterator it;
-    std::string name_str( head->headinfo->name );
+    std::list< std::string >::const_iterator it;
+    const std::string name_str( head->headinfo->name );
 
     // ローカル name
     if( check_name ){
@@ -2476,22 +2627,44 @@ bool NodeTreeBase::check_abone_name( int number )
 }
 
 
+//
+// number番のあぼーん判定( mail )
+//
+// あぼーんの時はtrueを返す
+//
+const bool NodeTreeBase::check_abone_mail( const int number )
+{
+    if( ! m_abone_age ) return false;
+
+    NODE* head = res_header( number );
+    if( ! head ) return false;
+    if( ! head->headinfo ) return false;
+    if( head->headinfo->abone ) return true;
+
+    if( ! head->headinfo->sage ){
+        head->headinfo->abone = true;        
+        return true;
+    }
+
+    return false;
+}
+
 
 //
 // number番のあぼーん判定( word, regex  /  board word, board regex / global word, global regex )
 //
 // あぼーんの時はtrueを返す
 //
-bool NodeTreeBase::check_abone_word( int number )
+const bool NodeTreeBase::check_abone_word( const int number )
 {
-    bool check_word = ! m_list_abone_word.empty();
-    bool check_regex = ! m_list_abone_regex.empty();
+    const bool check_word = ! m_list_abone_word.empty();
+    const bool check_regex = ! m_list_abone_regex.empty();
 
-    bool check_word_board = ! m_list_abone_word_board.empty();
-    bool check_regex_board = ! m_list_abone_regex_board.empty();
+    const bool check_word_board = ! m_list_abone_word_board.empty();
+    const bool check_regex_board = ! m_list_abone_regex_board.empty();
 
-    bool check_word_global = ! m_list_abone_word_global.empty();
-    bool check_regex_global = ! m_list_abone_regex_global.empty();
+    const bool check_word_global = ! m_list_abone_word_global.empty();
+    const bool check_regex_global = ! m_list_abone_regex_global.empty();
 
     if( !check_word && !check_regex
         && !check_word_board && !check_regex_board
@@ -2502,7 +2675,7 @@ bool NodeTreeBase::check_abone_word( int number )
     if( ! head->headinfo ) return false;
     if( head->headinfo->abone ) return true;
 
-    std::string res_str = get_res_str( number );
+    const std::string res_str = get_res_str( number );
     JDLIB::Regex regex;
 
     // ローカル NG word
@@ -2588,7 +2761,7 @@ bool NodeTreeBase::check_abone_word( int number )
 //
 // あぼーんしているレスにアンカーを張っているときはtrueを返す
 //
-bool NodeTreeBase::check_abone_chain( int number )
+const bool NodeTreeBase::check_abone_chain( const int number )
 {
     if( !m_abone_chain ) return false;
 
@@ -2780,7 +2953,7 @@ void NodeTreeBase::clear_id_name()
 //
 // from_number番から to_number 番までの発言数の更新
 //
-void NodeTreeBase::update_id_name( int from_number, int to_number )
+void NodeTreeBase::update_id_name( const int from_number, const int to_number )
 {
     if( ! CONFIG::get_check_id() ) return;
 
@@ -2794,53 +2967,66 @@ void NodeTreeBase::update_id_name( int from_number, int to_number )
 //
 // number番のレスの発言数を更新
 //
-void NodeTreeBase::check_id_name( int number )
+void NodeTreeBase::check_id_name( const int number )
 {
     NODE* header = res_header( number );
-
-    // あぼーんしているならチェックしない
+    if( ! header ) return;
     if( header->headinfo->abone ) return;
+    if( ! header->headinfo->block[ BLOCK_ID_NAME ] ) return;
 
-    if( header && header->headinfo->block[ BLOCK_ID_NAME ] ){
+    const char* str_id = header->headinfo->block[ BLOCK_ID_NAME ]->next_node->linkinfo->link;
 
-        char* str_id = header->headinfo->block[ BLOCK_ID_NAME ]->next_node->linkinfo->link;
+    // 同じIDのレスを持つ一つ前のレスを探す
+    NODE* tmphead;
+    NODE* prehead = NULL;
+    for( int i = header->id_header -1 ; i >= 1 ; --i ){
 
-        // 以前に出た同じIDのレスの発言数を更新しつつ発言回数を調べる
-        int num_id_name = 1;
-        for( int i = 1 ; i < header->id_header; ++i ){
+        tmphead = m_vec_header[ i ];
 
-            NODE* tmphead = m_vec_header[ i ];
-
-            if( tmphead
-                && ! tmphead->headinfo->abone // 対象スレがあぼーんしていたらカウントしない
-                && tmphead->headinfo->block[ BLOCK_ID_NAME ]
-                && strcmp( str_id, tmphead->headinfo->block[ BLOCK_ID_NAME ]->next_node->linkinfo->link ) == 0 ){
-
-                set_num_id_name( tmphead, tmphead->headinfo->num_id_name+1 );
-                ++num_id_name;
-            }
+        if( tmphead
+            && ! tmphead->headinfo->abone // 対象スレがあぼーんしていたらカウントしない
+            && tmphead->headinfo->block[ BLOCK_ID_NAME ]
+            && str_id[ 0 ] == tmphead->headinfo->block[ BLOCK_ID_NAME ]->next_node->linkinfo->link[ 0 ]
+            && strcmp( str_id, tmphead->headinfo->block[ BLOCK_ID_NAME ]->next_node->linkinfo->link ) == 0 ){
+            prehead = tmphead;
+            break;
         }
+    }
 
-        set_num_id_name( header, num_id_name );
+    // 見つからなかった
+    if( ! prehead ) set_num_id_name( header, 1, NULL );
+
+    // 見つかった
+    else{
+
+        set_num_id_name( header, prehead->headinfo->num_id_name+1, prehead );
+
+        // 以前に出た同じIDのレスの発言数を更新
+        tmphead = prehead;
+        while( tmphead ){
+
+            set_num_id_name( tmphead, tmphead->headinfo->num_id_name+1, tmphead->headinfo->pre_id_name_header );
+            tmphead = tmphead->headinfo->pre_id_name_header;
+        }
     }
 }
 
 
-
 //
 // 発言数( num_id_name )の更新
 //
 // IDノードの色も変更する
 //
-void NodeTreeBase::set_num_id_name( NODE* header, int num_id_name )
+void NodeTreeBase::set_num_id_name( NODE* header, const int num_id_name, NODE* pre_id_name_header )
 {
-    if( header->headinfo->block[ BLOCK_ID_NAME ] ){
+    if( ! header->headinfo->block[ BLOCK_ID_NAME ] ) return;
 
-        header->headinfo->num_id_name = num_id_name;        
-        if( num_id_name >= m_num_id[ LINK_HIGH ] ) header->headinfo->block[ BLOCK_ID_NAME ]->next_node->color_text = COLOR_CHAR_LINK_HIGH;
-        else if( num_id_name >= m_num_id[ LINK_LOW ] ) header->headinfo->block[ BLOCK_ID_NAME ]->next_node->color_text = COLOR_CHAR_LINK;
-        else header->headinfo->block[ BLOCK_ID_NAME ]->next_node->color_text = COLOR_CHAR;
-    }
+    header->headinfo->num_id_name = num_id_name;        
+    header->headinfo->pre_id_name_header = pre_id_name_header;
+
+    if( num_id_name >= m_num_id[ LINK_HIGH ] ) header->headinfo->block[ BLOCK_ID_NAME ]->next_node->color_text = COLOR_CHAR_LINK_HIGH;
+    else if( num_id_name >= m_num_id[ LINK_LOW ] ) header->headinfo->block[ BLOCK_ID_NAME ]->next_node->color_text = COLOR_CHAR_LINK;
+    else header->headinfo->block[ BLOCK_ID_NAME ]->next_node->color_text = COLOR_CHAR;
 }
 
 
