@@ -42,6 +42,7 @@ using namespace IMAGE;
 
 ImageViewBase::ImageViewBase( const std::string& url, const std::string& arg1, const std::string& arg2 )
     : SKELETON::View( url ),
+      m_wait( false ),
       m_loading( false ),
       m_under_mouse( false ),
       m_enable_menuslot( true )
@@ -297,10 +298,44 @@ void ImageViewBase::setup_common()
 
     "</popup>";
 
+    // 画像ポップアップのメニュー
+    const std::string menu_popup = 
+
+    "<popup name='popup_menu_popup'>"
+
+    "<menuitem action='CancelMosaic'/>"
+    "<separator/>"
+
+    "<menuitem action='Quit'/>"
+    "<separator/>"
+
+    "<menuitem action='CopyURL'/>"
+    "<separator/>"
+
+    "<menuitem action='Save'/>"
+    "<separator/>"
+
+    "<menuitem action='ProtectImage'/>"
+    "<menu action='DeleteMenu'>"
+    "<menuitem action='DeleteImage'/>"
+    "</menu>"
+
+    "<separator/>"
+    "<menuitem action='AboneImage'/>"
+
+    "<separator/>"
+
+    "<menuitem action='Preference'/>"
+
+    "</popup>"
+    ;
+
+
     ui_manager()->add_ui_from_string(
         "<ui>"
         + menu
         + menu_icon
+        + menu_popup
         + "</ui>"
         );
 
@@ -310,6 +345,9 @@ void ImageViewBase::setup_common()
 
     popupmenu = dynamic_cast< Gtk::Menu* >( ui_manager()->get_widget( "/popup_menu_icon" ) );
     CONTROL::set_menu_motion( popupmenu );
+
+    popupmenu = dynamic_cast< Gtk::Menu* >( ui_manager()->get_widget( "/popup_menu_popup" ) );
+    CONTROL::set_menu_motion( popupmenu );
 }
 
 
@@ -366,7 +404,13 @@ void ImageViewBase::reload()
 
     std::string refurl = m_img->get_refurl();
     const bool mosaic = m_img->get_mosaic();
-    m_img->download_img( refurl, mosaic );
+    m_img->download_img( refurl, mosaic, 0 );
+
+    if( m_img->is_loading() ){
+
+        CORE::core_set_command( "redraw", get_url() );
+        CORE::core_set_command( "redraw_article" );
+    }
 }
 
 
@@ -480,6 +524,8 @@ void ImageViewBase::slot_close_all_views()
 //
 void ImageViewBase::show_preference()
 {
+    CORE::core_set_command( "hide_popup" );
+
     SKELETON::PrefDiag* pref= CORE::PrefDiagFactory( get_parent_win(), CORE::PREFDIAG_IMAGE, get_url() );
 
     IMAGE::get_admin()->set_command_immediately( "disable_fold_win" ); // run 直前に呼ぶこと
@@ -495,10 +541,45 @@ void ImageViewBase::show_preference()
 //
 void ImageViewBase::delete_view()
 {
+    delete_view_impl( false );
+}
+
+
+void ImageViewBase::delete_view_impl( const bool show_diag )
+{
+    CORE::core_set_command( "hide_popup" );
+
+    if( show_diag ){
+
+        if( !m_img->is_protected() ){
+
+            SKELETON::MsgDiag mdiag( get_parent_win(),
+                                     "画像を削除しますか？", false, Gtk::MESSAGE_QUESTION, Gtk::BUTTONS_YES_NO );
+            mdiag.set_default_response( Gtk::RESPONSE_NO );
+            if( mdiag.run() != Gtk::RESPONSE_YES ) return;
+        }
+        else{
+
+            SKELETON::MsgDiag mdiag( get_parent_win(), "キャッシュ保護されています" );
+            mdiag.run();
+            return;
+        }
+    }
+
     CORE::core_set_command( "delete_image", get_url() );
 }
 
 
+//
+// クリックした時の処理
+//
+void ImageViewBase::clicked()
+{
+    IMAGE::get_admin()->set_command( "switch_image", get_url() );
+    IMAGE::get_admin()->set_command( "switch_admin" );
+}
+
+
 
 //
 // viewの操作
@@ -625,20 +706,7 @@ const bool ImageViewBase::operate_view( const int control )
             break;
 
         case CONTROL::Delete:
-
-            if( !m_img->is_protected() ){
-
-                SKELETON::MsgDiag mdiag( get_parent_win(),
-                                         "画像を削除しますか？", false, Gtk::MESSAGE_QUESTION, Gtk::BUTTONS_YES_NO );
-                mdiag.set_default_response( Gtk::RESPONSE_NO );
-                if( mdiag.run() == Gtk::RESPONSE_YES ) delete_view();
-            }
-            else{
-
-                SKELETON::MsgDiag mdiag( get_parent_win(), "キャッシュ保護されています" );
-                mdiag.run();
-            }
-
+            delete_view_impl( true );
             break;
 
             // サイドバー表示/非表示
@@ -656,6 +724,11 @@ const bool ImageViewBase::operate_view( const int control )
             slot_favorite();
             break;
 
+            // URLを開くダイアログを表示
+        case CONTROL::OpenURL:
+            CORE::core_set_command( "show_openurl_diag" );
+            break;
+
         default:
             return false;
     }
@@ -670,7 +743,7 @@ const bool ImageViewBase::operate_view( const int control )
 //
 // キープレスイベント
 //
-bool ImageViewBase::slot_key_press( GdkEventKey* event )
+const bool ImageViewBase::slot_key_press( GdkEventKey* event )
 {
 #ifdef _DEBUG
     std::cout << "ImageViewBase::slot_key_press url = " << get_url() << std::endl;
@@ -702,11 +775,8 @@ bool ImageViewBase::slot_button_press( GdkEventButton* event )
     if( event->type == GDK_2BUTTON_PRESS ) m_dblclick = true; 
 
     // クリック
-    // 反応を良くするため slot_button_release() ではなくてここで処理する
-    if( get_control().button_alloted( event, CONTROL::ClickButton ) ){
-        IMAGE::get_admin()->set_command( "switch_image", get_url() );
-        IMAGE::get_admin()->set_command( "switch_admin" );
-    }
+    // 反応を良くするため slot_button_release() ではなくてボタンを押した時点で処理する
+    if( get_control().button_alloted( event, CONTROL::ClickButton ) ) clicked();
 
     return true;
 }
@@ -823,7 +893,6 @@ void ImageViewBase::slot_reload_force()
     m_img->set_code( HTTP_INIT );
     m_img->set_type( DBIMG::T_UNKNOWN );
     reload();
-    CORE::core_set_command( "redraw", get_url() );
 }
 
 
@@ -1022,12 +1091,13 @@ void ImageViewBase::slot_copy_url()
 
 
 //
-// 保存メニュー
+// 保存
 //
 void ImageViewBase::slot_save()
 {
     if( ! m_enable_menuslot ) return;
 
+    CORE::core_set_command( "hide_popup" );
     m_img->save( IMAGE::get_admin()->get_win(), std::string() );
 }
 
@@ -1086,9 +1156,9 @@ void ImageViewBase::set_image_to_buffer()
 {
     CORE::DATA_INFO info;
     info.type = TYPE_IMAGE;
+    info.parent = IMAGE::get_admin()->get_win();
     info.url = get_url();
     info.name = get_url();
-    info.data = std::string();
     info.path = Gtk::TreePath( "0" ).to_string();
 
     CORE::DATA_INFO_LIST list_info;
