@@ -35,6 +35,20 @@ Iconv::Iconv( const std::string& coding_from, const std::string& coding_to )
         else if( coding_from == "MS932" ) m_cd = iconv_open( coding_to.c_str(), "CP932" ); 
     }
 
+    // "EUCJP-*"で失敗したら"EUCJP"で試してみる
+    if( m_cd == ( iconv_t ) - 1 && ( errno & EINVAL ) != 0 )
+    {
+        if( coding_to.find( "EUCJP-", 0 ) == 0 )
+		{
+            m_cd = iconv_open( "EUCJP//TRANSLIT", coding_from.c_str() );
+        }
+        else if( coding_from.find( "EUCJP-", 0 ) == 0 )
+		{
+            const std::string coding_to_translit = coding_to + "//TRANSLIT";
+            m_cd = iconv_open( coding_to_translit.c_str(), "EUCJP" );
+        }
+    }
+
     if( m_cd == ( iconv_t ) -1 ){
         MISC::ERRMSG( "can't open iconv coding = " + m_coding_from + " to " + coding_to );
     }
